using System;
using System.Drawing;
using System.Windows.Forms;

namespace VSTrainer
{
	/// <summary>
	/// "Checkbox Fighter", a tiny game by Thiemo 'mados' Mttig,
	/// mixed of "Space Invaders" and "Moorhuhn".
	/// </summary>
	public class CheckboxFighter : System.Windows.Forms.Form
	{
		private System.ComponentModel.IContainer components;
		private System.Windows.Forms.Timer timer_CheckboxFighter;

		// Some quasi global variables to speed up the code below a bit.
		private Point[] ship = new Point[3];
		private Pen shipPen = new Pen(Color.Blue, 5);
		private Point shot = new Point(-1, -1);
		private Pen shotPen = new Pen(Color.FromArgb(0, 255, 0), 3);
		private Point[] blasts = new Point[10];
		private int blastIterator = 0;
		private Pen blastPen = new Pen(Color.Red, 2);
		
		private Point[] targets = new Point[100];
		private Point[] targetsSpeed = new Point[100];
		private bool[] targetsChecked = new bool[100];
		private int targetCount = 1;

		private Point[] stars = new Point[50];
		private Pen starPen = new Pen(Color.Yellow, 2);

		private int steps = 0;
		private Font stepsFont = new Font(FontFamily.GenericMonospace, 9, FontStyle.Bold);
		private int score = 0;
		private Font scoreFont = new Font(FontFamily.GenericMonospace, 11, FontStyle.Bold);

		// Pens and brushes used mainly in the drawCheckbox method.
		private Pen penWhite = new Pen(Color.White);
		private Pen penLightGray = new Pen(Color.LightGray);
		private Pen penGray = new Pen(Color.Gray);
		private Pen penBlack = new Pen(Color.Black);
		private Pen penBlack2 = new Pen(Color.Black, 2);
		private Brush brushBlack = new SolidBrush(Color.Black);
		private Brush brushWhite = new SolidBrush(Color.White);

		// Create a single random generator object for use below.
		Random rnd = new Random();

		/// <summary>
		/// Default constructor including some initialisations.
		/// </summary>
		public CheckboxFighter()
		{
			InitializeComponent();

			// Enable the whole double buffering stuff.
			this.SetStyle(ControlStyles.UserPaint |
				ControlStyles.AllPaintingInWmPaint |
				ControlStyles.DoubleBuffer, true);

			// Initialize the positions of all background stars.
			for (int i = 0; i < stars.Length; i++)
			{
				stars[i] = new Point(rnd.Next(240), rnd.Next(320));
			}
			// Initialize the position of your star ship.
			ship[0] = new Point(120, 250);
			// Create the first target.
			targets[0] = new Point(120, -10);
			targetsSpeed[0] = new Point(0, 1);
		}

		/// <summary>
		/// Default destructor.
		/// </summary>
		protected override void Dispose(bool disposing)
		{
			if (disposing)
			{
				if (components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose(disposing);
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Erforderliche Methode fr die Designeruntersttzung. 
		/// Der Inhalt der Methode darf nicht mit dem Code-Editor gendert werden.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			this.timer_CheckboxFighter = new System.Windows.Forms.Timer(this.components);
			// 
			// timer_CheckboxFighter
			// 
			this.timer_CheckboxFighter.Enabled = true;
			this.timer_CheckboxFighter.Interval = 40;
			this.timer_CheckboxFighter.Tick += new System.EventHandler(this.CheckboxFighter_Tick);
			// 
			// CheckboxFighter
			// 
			this.AutoScale = false;
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.BackColor = System.Drawing.Color.Black;
			this.CausesValidation = false;
			this.ClientSize = new System.Drawing.Size(240, 320);
			this.Cursor = System.Windows.Forms.Cursors.Cross;
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedToolWindow;
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.Name = "CheckboxFighter";
			this.Opacity = 0.6600000262260437;
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
			this.Text = "Checkpoint Fighter";
			this.MouseDown += new System.Windows.Forms.MouseEventHandler(this.CheckboxFighter_OnMouseDown);
			this.MouseMove += new System.Windows.Forms.MouseEventHandler(this.CheckboxFighter_MouseMove);

		}
		#endregion

		/// <summary>
		/// Default main method.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new CheckboxFighter());
		}

		/// <summary>
		/// This timer tick handler will be called 25 times per second.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void CheckboxFighter_Tick(object sender, System.EventArgs e)
		{
			// Call the OnPaint method 25 times per second.
			this.Refresh();
		}

		/// <summary>
		/// This handler will be called when the mouse position changes.
		/// </summary>
		/// <param name="sender">The object which causes the event.</param>
		/// <param name="e">The event itself.</param>
		private void CheckboxFighter_MouseMove(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			// Set the ship position to the position of your mouse cursor.
			ship[0].X = e.X;
			ship[0].Y = e.Y;
		}

		/// <summary>
		/// This handler will be called when a mouse button is pressed.
		/// </summary>
		/// <param name="sender">The object which caused the event.</param>
		/// <param name="e">The event itself.</param>
		private void CheckboxFighter_OnMouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			// You can not shoot again if the last shot is still on screen.
			if (shot.Y < 0)
			{
				// Set the shot position to the top of your star ship.
				shot = ship[0];
			}
		}

		/// <summary>
		/// We want to override the OnPaint method and redraw the form ourself.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		protected override void OnPaint(System.Windows.Forms.PaintEventArgs e)
		{
			e.Graphics.Clear(Color.Black);

			// Draw all the stars and move them.
			for (int i = 0; i < stars.Length; i++)
			{
				// Reset the position if a star hits the bottom screen edge.
				if (stars[i].Y > 320) stars[i].Y = 0;
				// Draw a tiny yellow line for each star.
				e.Graphics.DrawLine(starPen, stars[i].X, stars[i].Y, stars[i].X, stars[i].Y + 5);
				// Move the stars down using four different speeds.
				stars[i].Y += 1 + i / 13;
			}

			// Draw the red jet engine blast behind your star ship.
			for (int i = 0; i < blasts.Length; i++)
			{
				e.Graphics.DrawArc(blastPen, blasts[i].X - i / 2 - 1, blasts[i].Y, 2 + i, 2, -20, 220);
				// Move the blasts a bit down.
				blasts[i].Y += 4;
			}
			// Set one blast particle to the position of your star ship.
			blasts[blastIterator].X = ship[0].X;
			blasts[blastIterator].Y = ship[0].Y + 22;
			// Increase the iterator so next time another blast particle will be moved.
			blastIterator = ++blastIterator % blasts.Length;

			// Calculate the triangle for your star ship.
			ship[1].X = ship[0].X + 8;
			ship[2].X = ship[0].X - 8;
			ship[1].Y = ship[2].Y = ship[0].Y + 18;
			// Draw the blue triangle.
			e.Graphics.DrawPolygon(shipPen, ship);

			// Draw the shot you fired.
			e.Graphics.DrawArc(shotPen, shot.X - 2, shot.Y, 4, 8, 120, 300);
			// Move the shot up.
			shot.Y -= 16;

			// Create a new target if there are still less than 100 targets.
			// The target count will increase in a logarithmic manner (thanks to MadMan).
			if (targetCount < 100 && rnd.NextDouble() < (double)10 / (2250 - steps))
			{
				// Move the new target to the bottom of the screen.
				targets[targetCount].Y = 999;
				// Increase the target counter.
				targetCount++;
			}
			for (int i = 0; i < targetCount; i++)
			{
				// Move the target vertical.
				targets[i].X += targetsSpeed[i].X;
				// Bounce the target if it hits the left or right screen edge.
				if (targets[i].X < 0 || targets[i].X > 240)
				{
					// Invert the vertical speed of the target.
					targetsSpeed[i].X = -targetsSpeed[i].X;
				}
				// Move the target to the bottom of the screen.
				targets[i].Y += targetsSpeed[i].Y;
				// If the target leaves the screen.
				if (targets[i].Y > 320)
				{
					// Increase the score if the target was checked.
					if (targetsChecked[i]) score++;
					// Reset position, speed and checked state of the target.
					targets[i].X = rnd.Next(240);
					targets[i].Y = -10;
					targetsSpeed[i].X = rnd.Next(-2, 3);
					targetsSpeed[i].Y = rnd.Next(1, 5);
					targetsChecked[i] = false;
				}
				// If the target is hit by the shot currently on screen.
				if (Math.Abs(targets[i].Y - shot.Y) < 11 &&
					Math.Abs(targets[i].X - shot.X) < 8)
				{
					// Kick the target a little bit up.
					targets[i].Y -= 10;
					// Check the target.
					targetsChecked[i] = true;
					// Move the shot out of the screen.
					shot.Y = -100;
				}
					// If your star ship is hit by a target.
				else if (Math.Abs(targets[i].X - ship[0].X) < 8 &&
					Math.Abs(targets[i].Y - ship[0].Y) < 8)
				{
					// Remove the cross from the target.
					targetsChecked[i] = false;
					// Shake your star ship a little bit.
					ship[0].X += rnd.Next(-10, 11);
				}
				// Draw the target.
				drawCheckbox(e.Graphics, targets[i], targetsChecked[i]);
			}

			// Increase the step counter.
			steps++;
			string seconds = ((float)steps / 25).ToString("0.0s");
			// Draw a tiny shadow.
			e.Graphics.DrawString(seconds, stepsFont, brushBlack, 3, 3);
			// Calculate the seconds out of the steps and display them on screen.
			e.Graphics.DrawString(seconds, stepsFont, brushWhite, 2, 2);
			// Draw a tiny shadow.
			e.Graphics.DrawString(score.ToString(), scoreFont, brushBlack, 3, 16);
			// Display the number of the targets you hit successfully.
			e.Graphics.DrawString(score.ToString(), scoreFont, brushWhite, 2, 15);

			// If 1 minute and 30 seconds elapsed.
			if (steps >= 2250)
			{
				// Draw a tiny shadow.
				e.Graphics.DrawString("GAME OVER",
					new Font(FontFamily.GenericMonospace, 29, FontStyle.Bold),
					new SolidBrush(Color.Black), 8, 131);
				e.Graphics.DrawString("GAME OVER",
					new Font(FontFamily.GenericMonospace, 29, FontStyle.Bold),
					new SolidBrush(Color.Red), 7, 130);
				// Stop the timer.
				this.timer_CheckboxFighter.Stop();
			}
		}

		/// <summary>
		/// Draws a checked or unchecked box.
		/// </summary>
		/// <param name="g">Graphics object, where to draw the checkbox.</param>
		/// <param name="point">Position, where to draw the checkbox.</param>
		/// <param name="pointChecked">Should the box be crossed or not.</param>
		private void drawCheckbox(Graphics g, Point point, bool pointChecked)
		{
			// Put the coordinates in some shorter variables.
			int x = point.X;
			int y = point.Y;

			// Draw an empty box.
			g.DrawRectangle(penWhite, x - 6, y - 6, 12, 12);
			g.DrawRectangle(penGray, x - 6, y - 6, 11, 11);
			g.DrawRectangle(penLightGray, x - 5, y - 5, 10, 10);
			g.DrawRectangle(penBlack, x - 5, y - 5, 9, 9);
			g.FillRectangle(brushWhite, x - 4, y - 4, 9, 9);

			// Draw a cross inside the box if needed.
			if (pointChecked)
			{
				g.DrawLine(penBlack2, x - 3, y - 3, x + 3, y + 3);
				g.DrawLine(penBlack2, x - 3, y + 3, x + 3, y - 3);
			}
		}
	}
}